const expect = require('chai').expect;
const proxyquire = require('proxyquire');
const sinon = require('sinon');
const cocoStub = sinon.stub();
const apiV2Stub = sinon.stub();
const concordance = proxyquire('../src/concordance', {
	'./utils/coco': cocoStub,
	'./utils/capi-v2': apiV2Stub
});
const v2toTmeResponse = {
	concordances:	[ { concept:
		{ id: 'http://api.ft.com/things/4046bb37-833f-3a84-9077-d0f5fc012e82',
			apiUrl: 'http://api.ft.com/organisations/4046bb37-833f-3a84-9077-d0f5fc012e82' },
		identifier:
		{ authority: 'http://api.ft.com/system/FACTSET-EDM',
			identifierValue: '05HWPZ-E' } },
	{ concept:
		{ id: 'http://api.ft.com/things/4046bb37-833f-3a84-9077-d0f5fc012e82',
			apiUrl: 'http://api.ft.com/organisations/4046bb37-833f-3a84-9077-d0f5fc012e82' },
		identifier:
		{ authority: 'http://api.ft.com/system/FT-TME',
			identifierValue: 'TnN0ZWluX09OX0FGVE1fT05fMTk2NjAx-T04=' } },
	{ concept:
		{ id: 'http://api.ft.com/things/4046bb37-833f-3a84-9077-d0f5fc012e82',
			apiUrl: 'http://api.ft.com/organisations/4046bb37-833f-3a84-9077-d0f5fc012e82' },
		identifier:
		{ authority: 'http://api.ft.com/system/LEI',
			identifierValue: '213800QAUUUP6I445N30' } } ]
};
const tmeToV2Response = {
	concordances: [
		{
			concept:
			{
				id: 'http://api.ft.com/things/dbb0bdae-1f0c-11e4-b0cb-b2227cce2b52',
				apiUrl: 'http://test.api.ft.com/brands/dbb0bdae-1f0c-11e4-b0cb-b2227cce2b52'
			},
			identifier:
			{
				authority: 'http://api.ft.com/system/FT-TME',
				identifierValue: 'abcd'
			}
		}
	]
};

describe('Concordance transform', () => {

	beforeEach(() => {
		cocoStub.reset();
		apiV2Stub.reset();
	});

	describe('TME to V2 concordance', () => {

		it('returns the V2 id from a TME id - CoCo', () => {
			cocoStub.returns(Promise.resolve(tmeToV2Response));
			return concordance({id: 'abcd'})
				.then(res => {
					expect(res).to.equal('dbb0bdae-1f0c-11e4-b0cb-b2227cce2b52');
					expect(cocoStub.calledOnce).to.be.true;
					expect(apiV2Stub.called).to.be.false;
				});
		});

		it('returns the V2 id from a TME id - API V2', () => {
			cocoStub.returns(Promise.resolve());
			apiV2Stub.returns(Promise.resolve(tmeToV2Response));
			return concordance({id: 'abcd'})
				.then(res => {
					expect(res).to.equal('dbb0bdae-1f0c-11e4-b0cb-b2227cce2b52');
					expect(cocoStub.calledOnce).to.be.true;
					expect(apiV2Stub.calledOnce).to.be.true;
				});
		});

		it('if no concordance it returns without changing the req', () => {
			cocoStub.returns(Promise.resolve());
			apiV2Stub.returns(Promise.resolve());
			return concordance({id: 'abcd'})
				.then(res => {
					expect(res).to.be.undefined;
					expect(cocoStub.calledOnce).to.be.true;
					expect(apiV2Stub.calledOnce).to.be.true;
				});
		});

	});

	describe('V2 to TME concordance', () => {

		it('returns the TME id from a V2 id - CoCo', () => {
			cocoStub.returns(Promise.resolve(v2toTmeResponse));
			return concordance({id: '4046bb37-833f-3a84-9077-d0f5fc012e82'})
				.then(res => {
					expect(res).to.equal('TnN0ZWluX09OX0FGVE1fT05fMTk2NjAx-T04=');
					expect(cocoStub.calledOnce).to.be.true;
					expect(apiV2Stub.called).to.be.false;
				});
		});

		it('returns the TME id from a V2 id - API V2', () => {
			cocoStub.returns(Promise.resolve());
			apiV2Stub.returns(Promise.resolve(v2toTmeResponse));
			return concordance({id: '4046bb37-833f-3a84-9077-d0f5fc012e82'})
				.then(res => {
					expect(res).to.equal('TnN0ZWluX09OX0FGVE1fT05fMTk2NjAx-T04=');
					expect(cocoStub.calledOnce).to.be.true;
					expect(apiV2Stub.calledOnce).to.be.true;
				});
		});

		it('if no concordance it returns without changing the req', () => {
			cocoStub.returns(Promise.resolve());
			apiV2Stub.returns(Promise.resolve());
			return concordance({id: '4046bb37-833f-3a84-9077-d0f5fc012e82'})
				.then(res => {
					expect(res).to.be.undefined;
					expect(cocoStub.calledOnce).to.be.true;
					expect(apiV2Stub.calledOnce).to.be.true;
				});
		});

	});

});
